@@ -118,7 +118,7 @@ import Graphics.UI.Editor.MakeEditor
 import Graphics.UI.Editor.Simple (stringEditor, okCancelFields)
 import Control.Event (registerEvent)
 import Graphics.UI.Editor.Basics
-    (eventPaneName, GUIEventSelector(..))
+    (eventText, GUIEventSelector(..))
 import qualified Data.Set as  Set (unions, member)
 import Data.Set (Set(..))
 import Graphics.UI.Gtk.Gdk.Events (Event(..))
@@ -133,9 +133,10 @@ withoutGroupPrefix s = case groupPrefix `stripPrefix` s of
                             Nothing -> s
                             Just s' -> s'
 
+
 initGtkRc :: IO ()
-initGtkRc = return ()
-{--	rcParseString ("style \"leksah-close-button-style\"\n" ++
+#if MIN_VERSION_gtk(0,11,0)
+initGtkRc = rcParseString ("style \"leksah-close-button-style\"\n" ++
     "{\n" ++
     "  GtkWidget::focus-padding = 0\n" ++
     "  GtkWidget::focus-line-width = 0\n" ++
@@ -143,7 +144,9 @@ initGtkRc = return ()
     "  ythickness = 0\n" ++
     "}\n" ++
     "widget \"*.leksah-close-button\" style \"leksah-close-button-style\"")
---}
+#else
+initGtkRc = return ()
+#endif
 
 removePaneAdmin :: RecoverablePane alpha beta delta =>  alpha -> delta ()
 removePaneAdmin pane = do
@@ -625,11 +628,11 @@ groupNameDialog parent =  liftIO $ do
     lower                      <-   dialogGetActionArea dia
     (widget,inj,ext,_)         <-   buildEditor moduleFields ""
     (widget2,_,_,notifier)     <-   buildEditor okCancelFields ()
-    registerEvent notifier Clicked (Left (\e -> do
-            case eventPaneName e of
+    registerEvent notifier ButtonPressed (\e -> do
+            case eventText e of
                 "Ok"    ->  dialogResponse dia ResponseOk
                 _       ->  dialogResponse dia ResponseCancel
-            return e))
+            return e)
     boxPackStart upper widget PackGrow 7
     boxPackStart lower widget2 PackNatural 7
     widgetShowAll dia
@@ -647,7 +650,7 @@ groupNameDialog parent =  liftIO $ do
                             $ emptyParams)
                     id
                     (\ a b -> a)
-            (stringEditor (\s -> True))]
+            (stringEditor (const True) True)]
 
 viewNest :: PaneMonad alpha => String -> alpha ()
 viewNest group = do
